/**
 * Created by dave on 16/1/28.
 */
'use strict';
var moment = require('moment');
var crypto = require('crypto');
var mysql  = require('@gopalroy/mysql-pool');
var Utils  = require('./Utils');

var Driver = {name: 'Driver', TokenKey: '``ICCD@2016``'},
    self   = Driver;
module.exports = self;

//使用openId创建token
self.makeToken =
function(openId) {
    var token,
        cipher = crypto.createCipher('aes128', new Buffer(self.TokenKey));

    cipher.setAutoPadding(true);
    token = cipher.update(JSON.stringify({openId: openId}), 'utf8', 'base64');
    token += cipher.final('base64');
    return token;
};
self.fromIdG =
function*(id) {
    var sql = 'SELECT F.*, S.name AS s4_name FROM t_staff F JOIN t_4s S ON F.s4_id = S.id WHERE F.id = ? AND S.biz_mode = 1 LIMIT 1';
    return (yield mysql.pool.queryP(sql, [id]))[0];
};
//根据openId获取司机信息
self.fromOpenIdG =
function*(openId) {
    var sql = 'SELECT F.*, S.name AS s4_name FROM t_staff F JOIN t_4s S ON F.s4_id = S.id WHERE F.wx_openid = ? AND S.biz_mode = 1 LIMIT 1';
    return (yield mysql.pool.queryP(sql, [openId]))[0];
};
//根据token获取司机信息
self.fromTokenG =
function*(token) {
    var json,
        decipher = crypto.createDecipher('aes128', new Buffer(self.TokenKey));

    decipher.setAutoPadding(true);
    try {
        json = decipher.update(token, 'base64', 'utf8');
        json += decipher.final('utf8');
        json = JSON.parse(json);
    } catch(ex) {
        throw new Error('无效token: ' + token);
    }
    return yield self.fromOpenIdG(json.openId);
};
self.updateG =
function*(filter, fields) {
    yield Utils.updateG('t_staff', filter, fields);
};
//根据手机号获取司机信息
self.fromPhoneG =
function*(phone) {
    var sql = 'SELECT F.*, S.name AS s4_name FROM t_staff F JOIN t_4s S ON F.s4_id = S.id WHERE F.phone = ? AND S.biz_mode = 1';
    return (yield mysql.pool.queryP(sql, [phone]))[0];
};
//绑定微信openId
self.bindOpenIdG =
function*(driverId, openId) {
    var sql = 'UPDATE t_staff SET wx_openid = ? WHERE id = ?';
    yield mysql.pool.queryP(sql, [openId, driverId]);
};
//分派车辆
self.dispatchCarG =
function*(driver, vehicleId) {
    var sql = 'INSERT t_disp_team SET ?';
    return yield mysql.pool.queryP(sql, [{fleet_id: driver.s4_id, team_id: 0, staff_id: driver.id,
                    vehicle_id: vehicleId, dispatch_time: moment().format('YYYY-MM-DD HH:mm:ss')}]);
};
//收回车辆
self.withdrawCarG =
function*(dispatch) {
    var sqlUpdate = 'UPDATE t_disp_team SET recover_time = ? WHERE id = ?',
        sqlInsert = 'INSERT t_disp_stat SET ?',
        dispatchStat;
    dispatch.recover_time = moment().format('YYYY-MM-DD HH:mm:ss');
    dispatchStat = yield self.getDispatchStatG(dispatch);
    yield mysql.pool.queryP(sqlUpdate, [dispatch.recover_time, dispatch.id]);
    yield mysql.pool.queryP(sqlInsert, [dispatchStat]);
    return dispatchStat;
};
self.getCarByIdG =
function*(id) {
    return (yield mysql.pool.queryP('SELECT * FROM t_car WHERE id = ?', [id]))[0];
};
self.getCarByLicenseG =
function*(license) {
    return (yield mysql.pool.queryP('SELECT * FROM t_car WHERE license = ?', [license]))[0];
};
self.getCarByObdCode =
function*(obdCode) {
    return (yield mysql.pool.queryP('SELECT * FROM t_car WHERE obd_code = ?', [obdCode]))[0];
};
//获取用车记录
self.getDispatchesG =
function*(filter, pageSize, nextMarker) {
    if (filter.id) {
        filter['D.id'] = filter.id;
        delete filter.id;
    }
    var sql   = `SELECT D.*, C.license, C.brand AS brandCode, C.series AS seriesCode, C.obd_code, CD.brand, CD.series,
                    T.trip_count, T.total_drive_time, T.total_mileage, T.total_oil
                 FROM t_disp_team D JOIN t_car C ON C.id = D.vehicle_id
                 LEFT JOIN t_disp_stat T ON D.id = T.id
                 LEFT JOIN t_car_dictionary CD ON C.brand = CD.brandCode AND C.series = CD.seriesCode`,
        query = Utils.makeQuerySql(sql, filter, 'dispatch_time-', pageSize, nextMarker);
    return yield mysql.pool.queryP(query.sql, query.args);
};
//获取用车记录条数
self.getDispatchesCountG =
function*(filter) {
    var query = Utils.assemblyPagingSQL('t_disp_team', filter);
    return (yield mysql.pool.queryP(query.sqlCount, query.args))[0].rowsCount;
};
//获取某次用车中的油耗,行程统计数据
self.getDispatchStatG =
function*(dispatch) {
    var sql = `SELECT ? AS id, COUNT(id) AS trip_count, SUM(runTime) AS total_drive_time, SUM(currentMileage)/1000 AS total_mileage,
                    SUM(currentAvgOilUsed*currentMileage/1000) AS total_oil
               FROM t_obd_drive WHERE obdCode = ? AND fireTime > ? AND fireTime < ?`;
    return (yield mysql.pool.queryP(sql, [dispatch.id, dispatch.obd_code, dispatch.dispatch_time, dispatch.recover_time || moment().format('YYYY-MM-DD HH:mm:ss')]))[0];
};
//获取某次用车中的行车记录
self.getTripsG =
function*(dispatch, pageSize, nextMarker) {
    var query   = Utils.assemblyPagingSQL('t_obd_drive', {obdCode: dispatch.obd_code, fireTime: '>' + dispatch.dispatch_time,
                    flameOutTime: '<' + dispatch.recover_time}, 'fireTime-', pageSize, nextMarker);
    return yield mysql.pool.queryP(query.sql, query.args);
};
//获取某次用车记录中的行车记录条数
self.getTripsCountG =
function*(dispatch) {
    var query   = Utils.assemblyPagingSQL('t_obd_drive', {obdCode: dispatch.obd_code, fireTime: '>' + dispatch.dispatch_time,
                    flameOutTime: '<' + dispatch.recover_time});
    return (yield mysql.pool.queryP(query.sqlCount, query.args))[0].rowsCount;
};
self.getTripByIdG =
function*(id) {
    return (yield mysql.pool.queryP('SELECT * FROM t_obd_drive WHERE id = ?', [id]))[0];
};
