/**
 * Created by dave on 16/1/25.
 */
'use strict';
var co     = require('co');
var moment = require('moment');
var Utils  = require('./Utils');
var Qrcode = require('./Qrcode');
var Driver = require('./Driver');
var Weixin = require('./Weixin');

var MsgActions = {name: 'MsgActions'},
    self       = MsgActions;
module.exports = self;

self.onSendVerifyCode =
function(msg, res) {
    var phone = msg.Content.substr(3, 11),
        code  = String(Math.random()).substr(2, 6);

    if (global.UNIT_TEST) {
        code = '123456';
    }

    co(function*() {
        if (!/^\d{11}$/.test(phone)) {
            msg.Content = '输入格式有误';
        } else {
            var driver = yield Driver.fromPhoneG(phone);
            if (!driver) {
                msg.Content = '系统没有你的信息,请联系管理人员';
            } else {
                if (!global.UNIT_TEST) {
                    yield Utils.sendSmsP(phone, "【英卡车队】你的验证码是：" + code + "，请不要把验证码泄露给其他人。");
                }
                Utils.addCacheData("YCCDV:" + msg.FromUserName, code, 15*60);// YCCDV: 英卡车队VerifyCode
                Utils.addCacheData("YCCDP:" + msg.FromUserName, phone, 15*60);// YCCDU: 英卡车队phone
                msg.Content = '验证码已发送至你的手机, 回复001加验证码完成绑定帐户,如001123456';
            }
        }
        self._reply(msg, res);
    }).catch(ex => {
        msg.Content = '发送验证码失败';
        self._reply(msg, res);
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack);
    });
};

self.onPrepareToBindAccount =
function(msg, res) {
    msg.Content = "回复000加手机号开始绑定帐户,如00018812345678";
    self._reply(msg, res);
};

self.onBindAccount =
function(msg, res) {
    var code = msg.Content.substr(3);

    co(function*() {
        var ccode = yield Utils.getCacheDataP("YCCDV:" + msg.FromUserName); // YCCDV: 英卡车队VerifyCode
        if (ccode != code) {
            msg.Content = '验证码错误';
        } else {
            var driver, phone = yield Utils.getCacheDataP("YCCDP:" + msg.FromUserName);
            if (phone) driver = yield Driver.fromPhoneG(phone);
            if (!driver) {
                msg.Content = '系统没有你的信息';
            } else {
                yield Driver.updateG({id: driver.id}, {wx_openid: msg.FromUserName});
                msg.Content = '成功绑定手机号:' + phone;
            }
        }
        self._reply(msg, res);
    }).catch(ex => {
        msg.Content = '系统错误';
        self._reply(msg, res);
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack);
    });
};

self.onScanToTakeCar =
function(msg, res) {
    var license = Qrcode.decodeCarLicense(msg.ScanCodeInfo.ScanResult);
    if (license) {
        Utils.addCacheData("YCCDL:" + msg.FromUserName, license, 15*60);
        msg.Content = "回复数字100确认出车:" + license;
    } else {
        msg.Content = "系统无此二维码";
    }
    self._reply(msg, res);
};

self.onTakeCar =
function(msg, res) {
    co(function*() {
        var driver  = yield Driver.fromOpenIdG(msg.FromUserName);
        var license = yield Utils.getCacheDataP("YCCDL:" + msg.FromUserName); // YCDDL:英卡车队License
        if (!driver) {
            msg.Content = '系统没有你的信息';
        } else if (!license) {
            msg.Content = '请先扫描车辆二维码';
        } else {
            var vehicle = yield Driver.getCarByLicenseG(license);
            if (!vehicle) {
                msg.Content = '无此车辆';
            } else if (vehicle.s4_id != driver.s4_id) {
                msg.Content = '非本车队车辆';
            } else {
                var dispatch1 = (yield Driver.getDispatchesG({vehicle_id: vehicle.id, recover_time: null}, 1, 0))[0];
                var dispatch2 = (yield Driver.getDispatchesG({staff_id: driver.id, recover_time: null}, 1, 0))[0];
                if (dispatch1) {
                    var t_driver = yield Driver.fromIdG(dispatch1.staff_id);
                    msg.Content = '司机“' + t_driver.nick + ', ' + t_driver.phone + '”正在使用此车';
                } else if (dispatch2) {
                    msg.Content = '你已借出车辆: ' + dispatch2.license;
                } else {
                    yield Driver.dispatchCarG(driver, vehicle.id);
                    msg.Content = "出车成功:" + license;
                }
            }
        }
        self._reply(msg, res);
    }).catch(ex => {
        msg.Content = '系统错误';
        self._reply(msg, res);
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack);
    });
};

self.onReturnCar =
function(msg, res) {
    co(function*() {
        var driver  = yield Driver.fromOpenIdG(msg.FromUserName);
        if (!driver) {
            msg.Content = '系统没有你的信息';
        } else {
            var dispatch = (yield Driver.getDispatchesG({staff_id: driver.id, recover_time: null}, 1, 0))[0];
            if (!dispatch) {
                msg.Content = '你没有未还车辆';
            } else {
                Utils.extend(dispatch, yield Driver.withdrawCarG(dispatch));
                yield Weixin.sendTemplateMsgForDispatchG(msg.FromUserName, dispatch);
                res.send('');
                return;
                //msg.Content = '成功归还车辆: ' + dispatch.license;
            }
        }
        self._reply(msg, res);
    }).catch(ex => {
        msg.Content = '系统错误';
        self._reply(msg, res);
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack);
    });
};

self.onGetStatus =
function(msg, res) {
    co(function*() {
        var dispatch,
            driver   = yield Driver.fromOpenIdG(msg.FromUserName);

        if (!driver) {
            msg.Content = '系统没有你的信息';
        } else if (!(dispatch = (yield Driver.getDispatchesG({staff_id: driver.id}, 1, 0))[0])) {
            msg.Content = '系统没有你的用车信息';
        } else {
            if (!dispatch.recover_time) {
                Utils.extend(dispatch, yield Driver.getDispatchStatG(dispatch));
            }
            yield Weixin.sendTemplateMsgForDispatchG(msg.FromUserName, dispatch);
            res.send('');
            return;
            //msg.Content = `车牌:${dispatch.license}, 出车时间: ${dispatch.dispatch_time}, 还车时间: ${dispatch.recover_time || '-'}, 行驶里程: ${dispatch.total_mileage || 0}公里, 耗油: ${dispatch.total_oil || 0}L`
        }
        self._reply(msg, res);
    }).catch(ex => {
        msg.Content = '系统错误';
        self._reply(msg, res);
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack);
    });
};

self.onGetProfile =
function(msg, res) {
    co(function*() {
        var driver = yield Driver.fromOpenIdG(msg.FromUserName);
        if (driver) msg.Content = [driver.s4_name, driver.nick, '手机:' + driver.phone, '驾照:' + (driver.license_type) + '/' + driver.license_num].join(', ');
        else msg.Content = '系统没有你的信息';
        self._reply(msg, res);
    }).catch(ex => {
        msg.Content = '系统错误';
        self._reply(msg, res);
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack);
    });
};

// 回复消息msg, _reply方法会自动调换FromUserName和ToUserName
self._reply =
function(msg, res) {
    var reply = "<xml><ToUserName><![CDATA[" + msg.FromUserName + "]]></ToUserName>" +
        "<FromUserName><![CDATA[" + msg.ToUserName + "]]></FromUserName>" +
        "<CreateTime>" + parseInt(new Date().getTime()/1000) + "</CreateTime>" +
        "<MsgType><![CDATA[text]]></MsgType>" +
        "<Content><![CDATA[" + msg.Content + "]]></Content>" +
        "<FuncFlag>0</FuncFlag></xml>";
    res.send(reply);
};