'use strict';
var AliMQS = require("ali-mns");
var redis  = require("redis");
var util   = require('util');
var Path = require("path");
var fs = require("fs");
var mysql  = require('@gopalroy/mysql-pool');
var redisPool  = require('@gopalroy/redis-pool');

var MQ_NAME_SMS = "MQ-SMS";

var aliCfg = (function(){
    var aliCfg = {
        accountId: "<your-account-id>",
        keyId: "<your-key-id>",
        keySecret: "<your-key-secret>",
        region: "<your-datacenter-region>"
    };
    // secret.json contains sensitive data, and will not be tracked by git
    var cfgPath = Path.join(Path.dirname(require.main.filename), "config/secret.js");
    if(fs.existsSync(cfgPath)){
        aliCfg = require(cfgPath);
    }
    else{
        console.warn("Missing 'secret.js' configuration file.");
    }

    return aliCfg;
})();

var aliAccount  = new AliMQS.Account(aliCfg.accountId,aliCfg.keyId,aliCfg.keySecret);

// var client = redis.createClient({host: "120.55.98.235",post: "6379", no_ready_check: true});
// client.auth("193ete0AD2936ac06c09&5307fab776Zcd0fatkf0c4ef");

var redisConf = redisPool.getRedis();
var client = redis.createClient({host: redisConf.host, post: redisConf.port , no_ready_check: true});
client.auth(redisConf.password);

module.exports = {
name: 'Utils',
/**
 * 发送短信
 * @param sim sim卡号
 * @param content 短信内容
 * @returns {Promise}
 */
sendSmsP:
function (sim, content) {
    var mqSms = new AliMQS.MQ(MQ_NAME_SMS, aliAccount, aliCfg.region);
    if (/[0-9]{11}/.test(sim)) {
        return mqSms.sendP(JSON.stringify({sim: sim, content: content, cmd: 0x1600}));
    } else {
        return Promise.reject(new Error('手机号错误'));
    }
},
/**
 * 写入redis缓存
 * @param key
 * @param data
 * @param expire 多少秒后失效,单位为秒
 */
addCacheData:
function(key, data, expire) {
    client.set(key, data);
    client.expire(key, expire);
},
/**
 * 获取缓存数据
 * @param key
 * @returns {Promise}
 */
getCacheDataP:
function(key) {
    return new Promise((resolve, reject) => {
        client.get(key, (err, reply) => {
            if (err) reject(err);
            else resolve(reply);
        });
    });
},
// 把由'/'引用起来的字符串转换为RegExp,其它返回原始字符串
parseRegExp:
function(strBuffer){
    if(strBuffer && strBuffer.length > 2
        && strBuffer[0] === '/'
        && strBuffer[strBuffer.length-1] === '/') {
        // 是正则表达式
        return new RegExp(strBuffer.substr(1, strBuffer.length-2));
    }
    else // 不是正则表达式
        return strBuffer;
},
// 把HTTP请求中的查询字符串,转换为批量查询条件
makeQueryArgs:
function (query) {
    var result = {
        filter: {}
    };

    if (query) {
        for (var k in query) {
            if (k === 'sortBy') result.sortBy = query[k];
            else if (k === 'pageSize') result.pageSize = Number(query[k]);
            else if (k === 'nextMarker') result.nextMarker = Number(query[k]);
            else {
                if (query[k] === "null") result.filter[k] = null;// null被解释为特别含义
                else result.filter[k] = this.parseRegExp(query[k]);
            }
        }
    }

    return result;
},
// 支持正则表达式的高级查询过滤条件
makeWhereClause:
function (filter) {
    var sql  = "";
    var args = [];
    if (filter) {
        for (let field in filter) {
            args.push(field);
            if (args.length > 1) sql += " and ";
            //判断过滤条件是否为正则表达式
            if (filter[field] instanceof RegExp) {
                sql += "?? REGEXP ?";
                // 由于JS的正则表达式toString()后前后各有一个'/',而mysql的不需要,因此只保留中间部分
                var strX = filter[field].toString();
                args.push(strX.substring(1, strX.lastIndexOf('/')));
            } else {
                if (filter[field] === null) {
                    sql += "?? is null";
                } else if (typeof filter[field] === "string") {
                    if (filter[field].substr(0, 2) === "<>") {
                        sql += "?? <> ?";
                        args.push(filter[field].substr(2));
                    } else if (filter[field].substr(0, 2) === "><") {
                        sql += "?? between ? and ?";
                        var num = filter[field].substr(2).split(',');
                        args.push(num[0]);
                        args.push(num[1]);
                    } else if (filter[field].substr(0, 2) === "<=") {
                        sql += "?? <= ?";
                        args.push(filter[field].substr(2));
                    } else if (filter[field].substr(0, 2) === ">=") {
                        sql += "?? >= ?";
                        args.push(filter[field].substr(2));
                    } else if (filter[field][0] === "<") {
                        sql += "?? < ?";
                        args.push(filter[field].substr(1));
                    } else if (filter[field][0] === ">") {
                        sql += "?? > ?";
                        args.push(filter[field].substr(1));
                    } else {
                        sql += "?? = ?";
                        args.push(filter[field]);
                    }
                } else {
                    sql += "?? = ?";
                    args.push(filter[field]);
                }
            }
        }
    }
    return {where: sql, args: args};
},
// 组装支持分页的SQL
assemblyPagingSQL:
function (tableName, filter, sortBy, pageSize, nextMarker) {
    // select & from
    var sqlTemplate      = "SELECT * FROM(SELECT P.*,@rowNum:=@rowNum+1 AS rowNo\n" +
        "FROM (SELECT T.*\n" +
        "\tFROM ?? AS T\n" +
        "\t%s%s) AS P JOIN (SELECT @rowNum := 0) R) U";
    var sqlCountTemplate = "SELECT count(*) AS rowsCount FROM ?? %s";
    var args             = [tableName];
    // where
    var sqlWhere    = "";
    var whereClause = this.makeWhereClause(filter);
    if (whereClause.args.length > 0) {
        sqlWhere = "WHERE " + whereClause.where;
        for (var i in whereClause.args) args.push(whereClause.args[i]);
    }
    // order by
    var sqlOrderBy = "";
    if (sortBy) {
        sqlOrderBy   = "\n\tORDER BY ??";
        var lastChar = sortBy.charAt(sortBy.length - 1);
        if (lastChar === "+" || lastChar === "-") {
            args.push(sortBy.substr(0, sortBy.length - 1));
            if (lastChar === "-") sqlOrderBy += " DESC"
        } else {
            args.push(sortBy);
        }
    }
    // where & order by
    var sql      = util.format(sqlTemplate, sqlWhere, sqlOrderBy);
    var sqlCount = util.format(sqlCountTemplate, sqlWhere);
    // limit
    if (!isNaN(pageSize) && pageSize >= 0) {
        if (nextMarker) {
            sql += "\nLIMIT ?,?";
            args.push(nextMarker - 1);
            args.push(pageSize);
        } else {
            sql += "\nLIMIT ?";
            args.push(pageSize);
        }
    } else {
        // 允许pageSize传-1,这时表示返回所有余下的行数
        if (nextMarker) {
            sql += "\nWHERE U.rowNo >= ?";
            args.push(nextMarker);
        }
    }

    return {sql: sql, sqlCount: sqlCount, args: args};
},
/**
 * 获取数据表的字段名称数组
 * @param table
 * @returns {*}
 */
getTableFieldsG:
function*(table) {
    var fields = [],
        result = yield mysql.pool.queryP("SHOW COLUMNS FROM ??", [table]);

    for (let row of result) {
        fields.push(row.Field);
    }
    return fields;
},
/**
 * 根据table表中的字段向输出对象out添加过滤条件
 * @param out 输出的过滤条件数组
 * @param table 表名
 * @param alias 过滤条件将要使用的表别名
 * @param filter 原过滤条件,属性名为字段名
 * @param sortBy 排序条件,若有有效值,输出的过滤条件中将添加sortBy属性
 * @param except 需要排除的属hash表
 */
mergeTableFilterG:
function*(out, table, alias, filter, sortBy, except) {
    var fields = yield this.getTableFieldsG(table),
        order;

    if (sortBy) {
        order = sortBy.charAt(sortBy.length - 1);
        if (order == '+' || order == '-') sortBy = sortBy.substr(0, sortBy.length - 1);
        else order = '+';
    }

    for (let f of fields) {
        if (filter[f] !== undefined && !except[f]) {
            out[alias + '.' + f] = filter[f];
            except[f] = true;
        }

        if (sortBy == f && !out.sortBy) {
            out.sortBy = alias + '.' + sortBy + order;
        }
    }
},
/**
 * 拼接sql语句.
 * @param sql 查询语句中select,from及join部分
 * @param filter
 * @param sortBy
 * @param pageSize
 * @param nextMarker
 * @returns {{sql: String, args: Array}}
 */
makeQuerySql:
function(sql, filter, sortBy, pageSize, nextMarker) {
    var args = [],
        whereClause = this.makeWhereClause(filter);

    if (whereClause.args.length > 0) {
        sql += " WHERE " + whereClause.where;
        args = whereClause.args
    }

    if (sortBy) {
        var lastChar = sortBy.charAt(sortBy.length - 1);
        if (lastChar === "+" || lastChar === "-") {
            sql += " ORDER BY " + sortBy.substr(0, sortBy.length - 1);
            if (lastChar === "-") sql += " DESC"
        } else {
            sql += " ORDER BY " + sortBy;
        }
    }

    if (!isNaN(pageSize) && pageSize >= 0) {
        if (nextMarker) {
            sql += " LIMIT " + (nextMarker - 1) + ", " + pageSize;
        } else {
            sql += " LIMIT " + pageSize;
        }
    }

    return {sql: sql, args: args};
},
updateG:
function*(table, filter, fields) {
    var sql  = "UPDATE ?? SET ",
        args = [table];

    for (let key in fields) {
        if (args > 1) sql += ",";
        sql += "?? = ?";
        args.push(key);
        args.push(fields[key]);
    }
    if (filter) {
        var query = this.makeWhereClause(filter);
        if (query.args.length > 0) {
            sql += ' WHERE ' + query.where;
            for (let a of query.args) {
                args.push(a);
            }
        }
    }
    yield mysql.pool.queryP(sql, args);
},
commonSelectG:
function*(table, filter, sortBy, pageSize, nextMarker) {
    var query = this.makeQuerySql('SELECT * FROM ' + table, filter, sortBy, pageSize, nextMarker);
    yield mysql.pool.queryP(query.sql, query.args);
},
commonCountG:
function*(table, filter) {
    var sql = 'SELECT COUNT(*) AS rowsCount FROM ?? ',
        query = this.makeWhereClause(filter);
    if (query.args.length > 1) {
        sql += ' WHERE ' + query.sql;
    }
    return (yield mysql.pool.queryP(sql, query.args))[0].rowsCount;
},
// 利用表结构信息进行插入
commonInsertG:
function*(table, object) {
    var sql     = "SHOW COLUMNS FROM ??;";
    var fSql    = " INSERT ?? SET ?";
    var columns = yield mysql.pool.queryP(sql, [table]);
    var newOrg  = {};
    for (let i = 0; i < columns.length; i++) {
        for (let field in object) {
            if (field == columns[i].Field) {
                newOrg[field] = object[field];
            }
        }
    }
    return yield mysql.pool.queryP(fSql, [table, newOrg]);
},
// 利用表结构信息进行更新
commonUpdateG:
function*(table, object) {
    var sql     = "SHOW COLUMNS FROM ??;";
    var uSql    = "UPDATE ?? SET ";
    var args    = [table];
    var columns = yield mysql.pool.queryP(sql, [table]);
    var flag    = false;
    for (let i = 0; i < columns.length; i++) {
        if (object[columns[i].Field] !== undefined) {
            if (flag) uSql += ",";
            flag = true;
            uSql += "??= ?";
            args.push(columns[i].Field);
            args.push(object[columns[i].Field]);
        }
    }
    if (flag) {
        uSql += " WHERE id = ?;";
        args.push(object.id);
        yield mysql.pool.queryP(uSql, args);
    }
},
//返回提取属性构成的Json对象
extract:
function(obj, props) {
    var ret = {};
    for (let i = 0; i < props.length; i++) {
        if (obj[props[i]] !== undefined) {
            ret[props[i]] = obj[props[i]];
        }
    }
    return ret;
},
// 向对象写入属性
extend:
function(obj, props) {
    Object.keys(props).forEach(k => {
        obj[k] = props[k];
    });
    return obj;
},
//从对象中删除属性
exclude:
function(obj, props) {
    Object.keys(props).forEach(k => {
        delete obj[k];
    });
}
};
