/**
 * Created by dave on 16/1/29.
 */
'use strict';
var co      = require('co');
var path    = require('path');
var express = require('express');
var moment  = require('moment');
var Weixin  = require('../lib').Weixin;
var Driver  = require('../lib').Driver;
var Utils   = require('../lib').Utils;

var DriverRouter = {context: '/driver'},
    self         = DriverRouter;
module.exports = self;

self.checkAuthority =
function(req, res, next) {
    var code  = req.query.code,
        token = req.cookies.token || req.query.token; // TODO 上线时删掉req.query.token
    co(function*() {
        if (token) {
            req.driver = yield Driver.fromTokenG(token);
            delete req.query.token; // TODO 上线时删掉
        } else if (code) {
            var openId = yield Weixin.getOpenIdG(code);
            if (openId) req.driver = yield Driver.fromOpenIdG(openId);
            delete req.query.code;
        } else {
            res.status(403).json({status: 'failed', error: 'NO_CREDENTIAL', message: '没有凭证信息'});
            return;
        }

        if (!req.driver) {
            if (req.path == '/' && req.method == 'GET') { // 首页页面总是允许进入
                next();
            } else {
                res.status(403).json({status: 'failed', error: 'NOT_AUTHORIZED_USER', message: '非授权用户'});
            }
        } else {
            res.cookie('token', Driver.makeToken(req.driver.wx_openid));
            next();
        }
    }).catch(ex => {
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack || ex);
        res.status(500).json({status: 'failed', error: 'SYSTEM_ERROR', message: ex.message});
    });
};

self.index =
function(req, res) {
    res.sendFile(path.resolve(__dirname + '/../static/driver.html'));
};

self.getDispatches =
function(req, res) {
    var args = Utils.makeQueryArgs(req.query);
    args.filter.staff_id = req.driver.id;
    co(function*() {
        var result = yield [Driver.getDispatchesCountG(args.filter),
                         Driver.getDispatchesG(args.filter, args.pageSize, args.nextMarker)];
        res.json({status: 'success', dispatches: result[1], count: result[1].length, total: result[0]});
    }).catch(ex => {
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack || ex);
        res.status(500).json({status: 'failed', error: 'SYSTEM_ERROR', message: ex.stack || ex});
    });
};

self.getTrips =
function(req, res) {
    var args = Utils.makeQueryArgs(req.query);
    co(function*() {
        var dispatch = (yield Driver.getDispatchesG({staff_id: req.driver.id, id: req.params.dispatchId}, 1, 0))[0];
        if (!dispatch || dispatch.staff_id != req.driver.id) {
            res.json({status: 'failed', error: 'ILLEGAL_PARAMS', message: '你没有此条用车记录'});
        } else {
            var result = yield [Driver.getTripsCountG(dispatch),
                                Driver.getTripsG(dispatch, args.pageSie, args.nextMarker)];
            res.json({status: 'success', trips: result[1], count: result[1].length, total: result[0]});
        }
    }).catch(ex => {
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack || ex);
        res.status(500).json({status: 'failed', error: 'SYSTEM_ERROR', message: ex.stack || ex});
    });
};

self.getTrip =
function(req, res) {
    co(function*() {
        var trip = yield Driver.getTripByIdG(req.params.id),
            car,
            dispatch;
        if (trip) car = yield Driver.getCarByObdCode(trip.obdCode);
        if (car) dispatch = (yield Driver.getDispatchesG(
            {
                dispatch_time: '<' + trip.fireTime,
                recover_time : '>' + trip.flameOutTime,
                vehicle_id   : car.id
            }, 1, null))[0];
        if (dispatch) {
            res.json({status: 'success', trip: trip});
        } else {
            res.json({status: 'failed', error: 'ILLEGAL_PARAMS', message: '你没有此条记录'});
        }
    }).catch(ex => {
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack || ex);
        res.status(500).json({status: 'failed', error: 'SYSTEM_ERROR', message: ex.stack || ex});
    });
};

self.getUserRouter =
function() {
    if (!self.userRouter) {
        self.userRouter = new express();
        self.userRouter.use('/',                                  this.checkAuthority);
        self.userRouter.get('/',                                  this.index);
        self.userRouter.get('/dispatch',                          this.getDispatches);
        self.userRouter.get('/dispatch/:dispatchId([0-9]+)/trip', this.getTrips);
        self.userRouter.get('/trip/:id([0-9]+)',                  this.getTrip);
    }
    return self.userRouter;
};