/**
 * Created by dave on 16/1/21.
 */
'use strict';
var co         = require('co');
var crypto     = require('crypto');
var request    = require('request');
var xml2js     = require('xml2js');
var express    = require('express');
var Weixin     = require('../lib').Weixin;
var MsgActions = require('../lib').MsgActions;

var WeixinRouter = {context: '/wechat'},
    self         = WeixinRouter;
module.exports = self;

self.xml2objP =
function(xml) {
    return new Promise((resolve, reject) => {
        xml2js.parseString(xml, {explicitArray: false, ignoreAttrs: true}, function(err, obj) {
            if (!err) resolve(obj.xml);
            else reject(err);
        });
    });
};
// 微信后台服务器验证
self.checkSignature =
function(req, res) {
    var encStr,
        query = req.query,
        str   = [Weixin.token, query.timestamp, query.nonce].sort().join('');

    encStr = crypto.createHash('sha1').update(str).digest('hex');
    if (encStr == query.signature) {
        res.send(query.echostr);
    } else {
        res.send(false);
    }
};

self.processMsg =
function(req, res) {
    co(function*() {
        var msg = yield self.xml2objP(req.body);
        if (msg.EventKey == 'menu_scan_to_take_car') {
            MsgActions.onScanToTakeCar(msg, res);
        } else if (msg.EventKey == 'menu_get_status') {
            MsgActions.onGetStatus(msg, res);
        } else if (msg.EventKey == 'menu_return_car') {
            MsgActions.onReturnCar(msg, res);
        } else if (msg.EventKey == 'menu_prepare_to_bind_account') {
            MsgActions.onPrepareToBindAccount(msg, res);
        } else if (msg.EventKey == 'menu_get_profile') {
            MsgActions.onGetProfile(msg, res);
        } else if (msg.MsgType == 'text' && msg.Content.substr(0, 3) == '000') {
            MsgActions.onSendVerifyCode(msg, res);
        } else if (msg.MsgType == 'text' && msg.Content.substr(0, 3) == '001') {
            MsgActions.onBindAccount(msg, res);
        } else if (msg.MsgType == 'text' && msg.Content == '100') {
            MsgActions.onTakeCar(msg, res);
        } else {
            res.send("");
        }
    }).catch(ex => {
        msg.Content = '系统错误';
        self._reply(msg, res);
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack || ex);
    });
};

self.createMenu =
function(req, res) {
    co(function*() {
        res.send(yield Weixin.createMenuG());
    }).catch(function(ex) {
        res.status(500).json({status: 'failed', error: 'SYSTEM_ERROR', message: ex.stack || ex});
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack || ex);
    });
};

self.getMenu =
function(req, res) {
    co(function*() {
        res.send(yield Weixin.getMenuG());
    }).catch(function(ex) {
        res.status(500).json({status: 'failed', error: 'SYSTEM_ERROR', message: ex.stack || ex});
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack || ex);
    });
};

self.deleteMenu =
function(req, res) {
    co(function*() {
        res.send(yield Weixin.deleteMenuG());
    }).catch(function(ex) {
        res.status(500).json({status: 'failed', error: 'SYSTEM_ERROR', message: ex.stack || ex});
        console.log('英卡车队@' + moment().format('MM-DD HH:mm:ss') + '[' + __filename + ']: ' + ex.stack || ex);
    });
};

// 回复消息msg, _reply方法会自动调换FromUserName和ToUserName
self._reply =
function(msg, res) {
    var reply = "<xml><ToUserName><![CDATA[" + msg.FromUserName + "]]></ToUserName>" +
        "<FromUserName><![CDATA[" + msg.ToUserName + "]]></FromUserName>" +
        "<CreateTime>" + parseInt(new Date().getTime()/1000) + "</CreateTime>" +
        "<MsgType><![CDATA[text]]></MsgType>" +
        "<Content><![CDATA[" + msg.Content + "]]></Content>" +
        "<FuncFlag>0</FuncFlag></xml>";
    res.send(reply);
};

self.getUserRouter =
function() {
    if (!self.userRouter) {
        self.userRouter = new express();
        self.userRouter.get ('/', self.checkSignature);
        self.userRouter.post('/', self.processMsg);
    }
    return self.userRouter;
};

self.getAdminRouter =
function() {
    if (!self.adminRouter) {
        self.adminRouter = new express();
        self.adminRouter.get('/createMenu', self.createMenu);
        self.adminRouter.get('/deleteMenu', self.deleteMenu);
        self.adminRouter.get('/getMenu',    self.getMenu);
    }
    return self.adminRouter;
};
