/**
 * Created by dave on 16/2/1.
 */
'use strict';
var co      = require('co');
var assert  = require('assert');
var request = require('request');
var xml2js  = require('xml2js');
var server  = require('..');
var testDb  = require('./TestDb');

global.UNIT_TEST = true;

function requestP(config) {
    return new Promise((resolve, reject) => {
        request(config, function(err, res) {
            if (err) reject(err);
            else resolve(res);
        });
    });
}

function formatMsg(from, to, msgType, content, eventKey) {
    return `<xml>
                <FromUserName>${from}</FromUserName>
                <ToUserName>${to}</ToUserName>
                <MsgType>${msgType}</MsgType>
                <EventKey>${eventKey}</EventKey>
                <Content>${content}</Content>
            </xml>`;

}

function xml2objP(xml) {
    return new Promise((resolve, reject) => {
        xml2js.parseString(xml, {explicitArray: false, ignoreAttrs: true}, function(err, obj) {
            if (!err) resolve(obj.xml);
            else reject(err);
        });
    });
}

describe('帐户绑定功能测试', function() {
    this.timeout(60000);
    var urlPrefix = 'http://localhost:8011/fleet-wechat/wechat';

    before(function(done) {
        co(function*() {
            yield testDb.initG();
        }).then(done, done);
    });

    after(function(done) {
        co(function*() {
            yield testDb.cleanG();
        }).then(done, done);
    });

    describe('菜单: 绑定帐户', function() {
        it('#menu menu_prepare_to_bind_account', function(done) {
            co(function*() {
                var res = yield requestP({
                    url    : urlPrefix,
                    method : 'POST',
                    body   : formatMsg(testDb.driver.wx_openid, testDb.org.openid, 'event', '', 'menu_prepare_to_bind_account'),
                    headers: {'Content-Type': 'text/xml'}
                });
                res = yield xml2objP(res.body);
                assert.equal(res.Content, '回复000加手机号开始绑定帐户,如00018812345678');
            }).then(done, done);
        });
    });

    describe('指令: 发送手机号开始绑定帐户', function() {
        it('#directive 000', function(done) {
            co(function*() {
                var res = yield requestP({
                    url    : urlPrefix,
                    method : 'POST',
                    body   : formatMsg(testDb.driver.wx_openid, testDb.org.openid, 'text', '000' + testDb.driver.phone),
                    headers: {'Content-Type': 'text/xml'}
                });
                res = yield xml2objP(res.body);
                assert.equal(res.Content, '验证码已发送至你的手机, 回复001加验证码完成绑定帐户,如001123456');
            }).then(done, done);
        });
    });

    describe('指定: 发送验证码完成帐户绑定', function() {
        it('#directive 001', function(done) {
            co(function*() {
                var res = yield requestP({
                    url    : urlPrefix,
                    method : 'POST',
                    body   : formatMsg(testDb.driver.wx_openid, testDb.org.openid, 'text', '001123456'),
                    headers: {'Content-Type': 'text/xml'}
                });
                res = yield xml2objP(res.body);
                assert.equal(res.Content, '成功绑定手机号:' + testDb.driver.phone);
            }).then(done, done);
        });
    });

    describe('菜单: 我的信息', function() {
        it('#menu menu_get_profile', function(done) {
            co(function*() {
                var res = yield requestP({
                    url    : urlPrefix,
                    method : 'POST',
                    body   : formatMsg(testDb.driver.wx_openid, testDb.org.openid, 'event', '', 'menu_get_profile'),
                    headers: {'Content-Type': 'text/xml'}
                });
                res = yield xml2objP(res.body);
                assert.equal(res.Content, [testDb.org.name, testDb.driver.nick, testDb.driver.phone].join(', '));
            }).then(done, done);
        });
    });

});