/**
 * Created by dave on 16/2/1 17:54.
 */
'use strict';
var co      = require('co');
var assert  = require('assert');
var request = require('request');
var xml2js  = require('xml2js');
var moment  = require('moment');
var mysql   = require('@gopalroy/mysql-pool');
var server  = require('..');
var testDb  = require('./TestDb');
var Qrcode  = require('../lib/Qrcode');

global.UNIT_TEST = true;
function requestP(config) {
    return new Promise((resolve, reject) => {
        request(config, function(err, res) {
            if (err) reject(err);
            else resolve(res);
        });
    });
}

function formatMsg(from, to, msgType, content, eventKey, other) {
    return `<xml>
                <FromUserName>${from}</FromUserName>
                <ToUserName>${to}</ToUserName>
                <MsgType>${msgType}</MsgType>
                <EventKey>${eventKey}</EventKey>
                <Content>${content}</Content>
                ${other || ''}
            </xml>`;

}

function xml2objP(xml) {
    return new Promise((resolve, reject) => {
        xml2js.parseString(xml, {explicitArray: false, ignoreAttrs: true}, function(err, obj) {
            if (!err) resolve(obj.xml);
            else reject(err);
        });
    });
}

describe('出车还车测试', function() {
    this.timeout(60000);
    var urlPrefix = 'http://localhost:8011/fleet-wechat/wechat';

    before(function(done) {
        co(function*() {
            yield testDb.initG();
        }).then(done, done);
    });

    after(function(done) {
        co(function*() {
            var disp_id = (yield mysql.pool.queryP('SELECT id from t_disp_team WHERE vehicle_id = ?', [testDb.car.id]))[0].id;
            yield mysql.pool.queryP('DELETE FROM t_disp_team WHERE vehicle_id = ?', testDb.car.id);
            yield mysql.pool.queryP('DELETE FROM t_disp_stat WHERE id = ?', [disp_id]);
            yield testDb.cleanG();
        }).then(done, done);
    });

    describe('菜单:扫描二维码', function() {
        it('menu menu_scan_to_take_car', function(done) {
            co(function*() {
                var res = yield requestP({
                    url    : urlPrefix,
                    method : 'POST',
                    body   : formatMsg(testDb.driver.wx_openid, testDb.org.openid, 'event', '', 'menu_scan_to_take_car',
                            `<ScanCodeInfo><ScanResult>${Qrcode.encodeCarLicense(testDb.car.license)}</ScanResult></ScanCodeInfo>`),
                    headers: {'Content-Type': 'text/xml'}
                });
                res = yield xml2objP(res.body);
                assert.equal(res.Content, '回复数字100确认出车:' + testDb.car.license);
            }).then(done, done);
        });
    });

    describe('指令:确认出车', function() {
        it('directive 100', function(done) {
            co(function*() {
                var res = yield requestP({
                    url    : urlPrefix,
                    method : 'POST',
                    body   : formatMsg(testDb.driver.wx_openid, testDb.org.openid, 'text', '100'),
                    headers: {'Content-Type': 'text/xml'}
                });
                res = yield xml2objP(res.body);
                assert.equal(res.Content, "出车成功:" + testDb.car.license)
            }).then(done, done);
        });
    });

    describe('菜单:还车', function() {
        it('menu menu_return_car', function(done) {
            co(function*() {
                var res = yield requestP({
                    url    : urlPrefix,
                    method : 'POST',
                    body   : formatMsg(testDb.driver.wx_openid, testDb.org.openid, 'event', '', 'menu_return_car'),
                    headers: {'Content-Type': 'text/xml'}
                });
                assert.equal(res.body, '');
            }).then(done, done);
        });
    });

    describe('菜单:获取用车状态', function() {
        it('menu menu_get_status', function(done) {
            co(function*() {
                var res = yield requestP({
                    url    : urlPrefix,
                    method : 'POST',
                    body   : formatMsg(testDb.driver.wx_openid, testDb.org.openid, 'event', '', 'menu_get_status'),
                    headers: {'Content-Type': 'text/xml'}
                });
                assert.equal(res.body, '');
            }).then(done, done);
        });
    });

    describe('菜单:扫描二维码2', function() {
        it('menu menu_scan_to_take_car', function(done) {
            co(function*() {
                var res = yield requestP({
                    url    : urlPrefix,
                    method : 'POST',
                    body   : formatMsg(testDb.driver.wx_openid, testDb.org.openid, 'event', '', 'menu_scan_to_take_car',
                        `<ScanCodeInfo><ScanResult>${Qrcode.encodeCarLicense('鄂A123BB')}</ScanResult></ScanCodeInfo>`),
                    headers: {'Content-Type': 'text/xml'}
                });
                res = yield xml2objP(res.body);
                assert.equal(res.Content, '回复数字100确认出车:鄂A123BB');
            }).then(done, done);
        });
    });

    describe('指令:确认出车2', function() {
        it('directive 100', function(done) {
            co(function*() {
                var res = yield requestP({
                    url    : urlPrefix,
                    method : 'POST',
                    body   : formatMsg(testDb.driver.wx_openid, testDb.org.openid, 'text', '100'),
                    headers: {'Content-Type': 'text/xml'}
                });
                res = yield xml2objP(res.body);
                assert.equal(res.Content, "无此车辆");
            }).then(done, done);
        });
    });
});
