/**
 * Created by dave on 16/2/2 16:29.
 */
'use strict';
var co      = require('co');
var assert  = require('assert');
var request = require('request');
var xml2js  = require('xml2js');
var moment  = require('moment');
var mysql   = require('@gopalroy/mysql-pool');
var Driver  = require('../lib').Driver;
var server  = require('..');
var testDb  = require('./TestDb');

global.UNIT_TEST = true;
function requestP(config) {
    return new Promise((resolve, reject) => {
        request(config, function(err, res) {
            if (err) reject(err);
            else resolve(res);
        });
    });
}

describe('司机数据记录', function() {
    this.timeout(60000);
    var urlPrefix = 'http://localhost:8011/fleet-wechat';
    var token = Driver.makeToken(testDb.driver.wx_openid);
    var dispatches;
    var trips;

    before(function(done) {
        co(function*() {
            yield testDb.initG();
            dispatches = [{
                fleet_id     : testDb.org.id,
                team_id      : 0,
                staff_id     : testDb.driver.id,
                vehicle_id   : testDb.car.id,
                dispatch_time: '2016-1-5',
                recover_time : '2016-1-9'
            }, {
                fleet_id     : testDb.org.id,
                team_id      : 0,
                staff_id     : testDb.driver.id,
                vehicle_id   : testDb.car.id,
                dispatch_time: '2016-1-18',
                recover_time : '2016-1-21'
            }];
            for(var d of dispatches) {
                d.id = (yield mysql.pool.queryP('INSERT t_disp_team SET ?', [d])).insertId;
            }

            trips = [{
                obdCode          : testDb.car.obd_code,
                fireTime         : '2016-1-6 12:12',
                flameOutTime     : '2016-1-6 13:29',
                currentMileage   : 87000,
                currentAvgOilUsed: 0.15,
                runTime          : 60 * 60
            }, {
                obdCode          : testDb.car.obd_code,
                fireTime         : '2016-1-7 10:10',
                flameOutTime     : '2016-1-7 11:41',
                currentMileage   : 18000,
                currentAvgOilUsed: 0.3,
                runTime          : 28 * 60
            }, {
                obdCode          : testDb.car.obd_code,
                fireTime         : '2016-1-18 13:12',
                flameOutTime     : '2016-1-18 14:29',
                currentMileage   : 87000,
                currentAvgOilUsed: 0.2,
                runTime          : 60 * 60
            }, {
                obdCode          : testDb.car.obd_code,
                fireTime         : '2016-1-19 15:12',
                flameOutTime     : '2016-1-19 16:29',
                currentMileage   : 88000,
                currentAvgOilUsed: 0.2,
                runTime          : 58 * 60
            }];
            for (var t of trips) {
                t.id = (yield mysql.pool.queryP('INSERT t_obd_drive SET ?', [t])).insertId;
            }
        }).then(done, done);
    });

    after(function(done) {
        co(function*() {
            yield mysql.pool.queryP('DELETE FROM t_obd_drive WHERE obdCode = ?', [testDb.car.obd_code]);
            yield mysql.pool.queryP('DELETE FROM t_disp_team WHERE vehicle_id = ?', [testDb.car.id]);
            yield testDb.cleanG();
        }).then(done, done);
    });

    describe('获取用车记录', function() {
        it('GET /driver/dispatch', function(done) {
            co(function*() {
                var res = yield requestP({
                    url   : urlPrefix + '/driver/dispatch',
                    method: 'GET',
                    qs    : {token: token, pageSize: 20, nextMarker: 1},
                    json  : true
                });
                assert.equal(res.body.total, 2);
                assert.equal(res.body.dispatches[0].vehicle_id, testDb.car.id);
            }).then(done, done);
        });
    });

    describe('获取行车记录', function() {
        it('GET /driver/dispatch/:dispatchId/trip', function(done) {
            co(function*() {
                var res = yield requestP({
                    url   : urlPrefix + '/driver/dispatch/' + dispatches[0].id + '/trip',
                    method: 'GET',
                    qs    : {token: token, pageSize: 20, nextMarker: 1},
                    json  : true
                });
                assert.ok(res.body.total > 0);
                assert.equal(res.body.trips[0].obdCode, testDb.car.obd_code);
            }).then(done, done);
        });
    });

    describe('获取行车详情', function() {
        it('GET /driver/trip/:tripId', function(done) {
            co(function*() {
                var res = yield requestP({
                    url   : urlPrefix + '/driver/trip/' + trips[0].id,
                    method: 'GET',
                    qs    : {token: token},
                    json  : true
                });
                assert.equal(res.body.status, 'success');
                assert.equal(res.body.trip.obdCode, testDb.car.obd_code);
            }).then(done, done);
        });
    });
});
